@@ -210,20 +210,18 @@ swf_object_get_tagdata(swf_object_t *swf, int tag_seqno,
         tag = tag->next;
     }
     if (tag) {
-        if (tag->data) {
-            *length = tag->length;
-            return tag->data;
-        }
-        if (tag->detail == NULL) {
+        if (tag->detail) {
             bitstream_t *bs;
-            unsigned char *data;
+            if (tag->data) {
+                free(tag->data);
+                tag->data = NULL;
+            }
             bs = bitstream_open();
             swf_tag_build(bs, tag, swf);
-            data = bitstream_steal(bs, length);
+            tag->data = bitstream_steal(bs, &(tag->length));
             bitstream_close(bs);
-            return data;
         }
-        if (tag->detail) {
+        if (tag->data) {
             *length = tag->length;
             return tag->data;
         }
@@ -231,6 +229,163 @@ swf_object_get_tagdata(swf_object_t *swf, int tag_seqno,
     return NULL;
 }
 
+int
+swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
+                           unsigned char *data, unsigned long length) {
+    int i;
+    swf_tag_t *tag;
+    tag = swf->tag;
+    for (i=0 ; (i < tag_seqno) &&  tag ; i++) {
+        tag = tag->next;
+    }
+    if (tag) {
+        if (tag->data) {
+            free(tag->data);
+        }
+        if (tag->detail) { // FIXME
+            swf_tag_destroy(tag);
+        }
+        tag->length = length;
+        tag->data = malloc(length);
+        memcpy(tag->data, data, length);
+        return 0;
+    }
+    return 1;
+}
+
+unsigned char *
+swf_object_get_tagcontents_bycid(swf_object_t *swf, int cid,
+                                  unsigned long *length) {
+    swf_tag_t *tag;
+    tag = swf->tag;
+    while (tag) {
+        if (swf_tag_identity(tag, cid) == 0) {
+            break; // match
+        }
+        tag = tag->next;
+    }
+    if (tag) {
+        if (tag->detail) {
+            bitstream_t *bs;
+            if (tag->data) {
+                free(tag->data);
+                tag->data = NULL;
+            }
+            bs = bitstream_open();
+            swf_tag_build(bs, tag, swf);
+            tag->data = bitstream_steal(bs, &(tag->length));
+            bitstream_close(bs);
+        }
+        if (tag->data) {
+            unsigned char *data;
+            *length = tag->length - 2;
+            return tag->data + 2;
+        }
+    }
+    *length = 0;
+    return NULL;
+}
+
+int
+swf_object_replace_tagcontents_bycid(swf_object_t *swf, int cid,
+                                     unsigned char *data,
+                                     unsigned long length) {
+    swf_tag_t *tag;
+    tag = swf->tag;
+    while (tag) {
+        if (swf_tag_identity(tag, cid) == 0) {
+            break; // match
+        }
+        tag = tag->next;
+    }
+    if (tag) {
+        if (tag->detail) {
+            swf_tag_destroy(tag);
+        }
+        if (tag->data) {
+            free(tag->data);
+            tag->data = NULL;
+        }
+        tag->length = length + 2;
+        tag->data = malloc(length + 2);
+        tag->data[0] = cid & 0xff;
+        tag->data[1] = cid >> 8;
+        memcpy(tag->data + 2, data, length);
+        return 0; // success
+    }
+    return 1; // failure
+}
+
+
+unsigned char *
+swf_object_get_shapedata(swf_object_t *swf, int cid, unsigned long *length) {
+    swf_tag_t *tag;
+    tag = swf->tag;
+    while (tag) {
+        if (swf_tag_identity(tag, cid) == 0) {
+            break; // match
+        }
+        tag = tag->next;
+    }
+    if (tag) {
+        if (! isShapeTag(tag->tag)) {
+            fprintf(stderr, "");
+            return NULL;
+        }
+        if (tag->detail) {
+            bitstream_t *bs;
+            if (tag->data) {
+                free(tag->data);
+                tag->data = NULL;
+            }
+            bs = bitstream_open();
+            swf_tag_build(bs, tag, swf);
+            tag->data = bitstream_steal(bs, &(tag->length));
+            bitstream_close(bs);
+        }
+        if (tag->data) {
+            unsigned char *data;
+            *length = tag->length - 2;
+            return tag->data + 2;
+        }
+    }
+    *length = 0;
+    return NULL;
+}
+
+int
+swf_object_replace_shapedata(swf_object_t *swf, int cid,
+                             unsigned char *data,
+                             unsigned long length) {
+    swf_tag_t *tag;
+    tag = swf->tag;
+    while (tag) {
+        if (swf_tag_identity(tag, cid) == 0) {
+            break; // match
+        }
+        tag = tag->next;
+    }
+    if (tag) {
+        if (! isShapeTag(tag->tag)) {
+            return 1; // failure
+        }
+        if (tag->detail) {
+            swf_tag_destroy(tag);
+        }
+        if (tag->data) {
+            free(tag->data);
+            tag->data = NULL;
+        }
+        tag->length = length + 2;
+        tag->data = malloc(length + 2);
+        tag->data[0] = cid & 0xff;
+        tag->data[1] = cid >> 8;
+        memcpy(tag->data + 2, data, length);
+        return 0; // success
+    }
+    return 1; // failure
+}
+
 /* --- */
 
 swf_tag_t *
@@ -326,13 +481,6 @@ swf_object_get_bitmap_size(swf_object_t *swf, int bitmap_id,
 
 /* --- */
 
-int
-swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
-                           unsigned char *data, unsigned long *length) {
-    fprintf(stderr, "swf_object_replace_tagdata: not impremented yet.\n");
-    return 1;
-}
-
 unsigned char *
 swf_object_get_jpegdata(swf_object_t *swf, unsigned long *length, int image_id) {
     swf_tag_t *tag, *tag_jpegtables = NULL;
